@@ -2,8 +2,6 @@
 #define DVIPNG_H
 #include "config.h"
 
-#define TIMING
-
 #define  STRSIZE         255     /* stringsize for file specifications  */
 
 #define  FIRSTFNTCHAR  0
@@ -15,8 +13,6 @@
 /* Name of the program which is called to generate missing pk files */
 #define MAKETEXPK "mktexpk"
 
-#define ERR_STREAM stderr   /* ???? */
-
 #ifdef HAVE_STDINT_H
 #include <stdint.h>
 #else
@@ -38,6 +34,12 @@
 #include <kpathsea/kpathsea.h>
 #endif
 
+#ifdef HAVE_FT2
+#include <freetype/freetype.h>
+FT_Library  libfreetype;
+#endif
+
+
 typedef  int     bool;
 #define  _TRUE      (bool) 1
 #define  _FALSE     (bool) 0
@@ -47,7 +49,7 @@ typedef  int     bool;
 # ifdef HAVE_DOPRNT
 #  define   vfprintf(stream, message, args)  _doprnt(message, args, stream)
 # else
-#  define vprintf vprintf_and_doprnt_missing
+#  error: vprintf AND _doprnt are missing!!!
    /* If we have neither, should fall back to fprintf with fixed args.  */
 # endif
 #endif
@@ -137,27 +139,43 @@ uint32_t         CommandLength(unsigned char*);
 /********************************************************/
 /***********************  font.h  ***********************/
 /********************************************************/
+struct encoding {
+  struct encoding* next;
+  char*            name;
+  char*            charname[256];
+};
+
 
 #define FONT_TYPE_PK            1
 struct pk_char {                   /* PK character */
-  unsigned char* mmap;             /* Points to beginning of PK data    */
-  uint32_t       length;           /* Length of PK data                 */
   int32_t        tfmw;             /* TFM width                         */
-  unsigned char  flag_byte;        /* PK flagbyte                       */
-  gdFont         glyph;            /* contains width and height in
-		                    * pixels, number of greylevels and
-		                    * the bitmaps for the greylevels
-				    */
+  unsigned char  greys;            /* ink is black at this value        */
+  int            w,h;              /* width height in pixels            */
+  char          *data;             /* glyph data                        */
   short          xOffset, yOffset; /* x offset and y offset in pixels   */
+  unsigned char *mmap;             /* Points to beginning of PK data    */
+  uint32_t       length;           /* Length of PK data                 */
+  unsigned char  flag_byte;        /* PK flagbyte                       */
 };
 
 #define FONT_TYPE_VF            2
 struct vf_char {                   /* VF character                     */
+  int32_t        tfmw;             /* TFM width                        */
   unsigned char* mmap;             /* Points to beginning of VF macro  */
   uint32_t       length;           /* Length of VF macro               */
-  int32_t        tfmw;             /* TFM width                        */
 };
 
+#ifdef HAVE_FT2
+#define FONT_TYPE_FT            3
+struct ft_char {                   /* FT character */
+  int32_t        tfmw;             /* TFM width                         */
+  unsigned char  greys;            /* ink is black at this value        */
+  int            w,h;              /* width height in pixels            */
+  char          *data;             /* glyph data                        */
+  short          xOffset, yOffset; /* x offset and y offset in pixels   */
+};
+#endif
+
 struct font_entry {    /* font entry */
   int          type;            /* PK/VF/Type1 ...                   */
   struct font_entry *next;
@@ -168,9 +186,14 @@ struct font_entry {    /* font entry */
   char         name[STRSIZE];   /* full name of PK/VF file           */
   int          filedes;         /* file descriptor                   */
   unsigned char* mmap;          /* memory map                        */
+  int          length;          /* its length                        */
   uint32_t     magnification;   /* magnification read from font file */
   uint32_t     designsize;      /* design size read from font file   */
   void *       chr[NFNTCHARS];  /* character information             */ 
+#ifdef HAVE_FT2
+  FT_Face      face;            /* Type1/Truetype font               */
+  struct encoding *enc;         /* custom encoding array             */
+#endif
   struct font_num *vffontnump;  /* VF local font numbering           */
   int32_t      defaultfont;     /* VF default font number            */
 };
@@ -189,6 +212,16 @@ void    InitVF (struct font_entry *);
 void    FontDef(unsigned char*, void* /* dvi/vf */);
 void    SetFntNum(int32_t, void* /* dvi/vf */);      
 
+#ifdef HAVE_FT2
+void    InitPSFontMap(void);
+char*   FindPSFontMap(char*, char**, FT_Matrix**);
+void    InitFT(struct font_entry *, unsigned, char*, FT_Matrix* );
+int32_t SetFT(int32_t, int32_t, int32_t);
+void    LoadFT(int32_t, struct ft_char *);
+struct encoding* FindEncoding(char*);
+void    ReadTFM(struct font_entry *, char*);
+#endif
+
 /********************************************************/
 /*********************  pplist.h  ***********************/
 /********************************************************/
@@ -231,7 +264,8 @@ void      CreateImage(void);
 void      DrawCommand(unsigned char*, void* /* dvi/vf */); 
 void      DrawPages(void);
 void      WriteImage(char*, int);
-void      LoadAChar(int32_t, register struct pk_char *);
+void      LoadPK(int32_t, register struct pk_char *);
+//void      LoadFT(int32_t, struct ft_char *);
 int32_t   SetChar(int32_t);
 int32_t   SetPK(int32_t,int32_t, int32_t);
 int32_t   SetVF(int32_t);
@@ -268,20 +302,26 @@ EXTERN struct internal_state {
   int                passno;
 } current_state;
 
-#define BE_NONQUIET 1
-#define BE_VERBOSE  2
-#define PARSE_STDIN 4
-#define LASTFLAG    PARSE_STDIN
+#define BE_NONQUIET     1
+#define BE_VERBOSE      2
+#define PARSE_STDIN     4
+#define REPORT_BASELINE 8
+#define LASTFLAG        REPORT_BASELINE
 
 #ifdef DEBUG
 /*EXTERN unsigned int Debug INIT(0);*/
 #define DEBUG_PRINT(a) Message a
-#define DEBUG_DVI   LASTFLAG * 2
-#define DEBUG_VF    LASTFLAG * 4
-#define DEBUG_PK    LASTFLAG * 8
-#define DEBUG_GLYPH LASTFLAG * 16
-#define DEBUG_GS    LASTFLAG * 32
-#define LASTDEBUG   DEBUG_GS
+#define DEBUG_DEFAULT LASTFLAG * 2
+#define DEBUG_DVI     DEBUG_DEFAULT
+#define DEBUG_VF      DEBUG_DEFAULT * 2
+#define DEBUG_PK      DEBUG_DEFAULT * 4
+#define DEBUG_TFM     DEBUG_DEFAULT * 8
+#define DEBUG_GLYPH   DEBUG_DEFAULT * 16
+#define DEBUG_FT      DEBUG_DEFAULT * 32
+#define DEBUG_ENC     DEBUG_DEFAULT * 64
+#define DEBUG_COLOR   DEBUG_DEFAULT * 128
+#define DEBUG_GS      DEBUG_DEFAULT * 256
+#define LASTDEBUG     DEBUG_GS
 #else
 #define DEBUG_PRINT(a)
 #endif
@@ -323,6 +363,9 @@ EXTERN struct timeb timebuffer;
 
 EXTERN int   resolution INIT(300);
 EXTERN char *MFMODE     INIT("cx");
+#ifdef HAVE_FT2
+EXTERN bool   freetype INIT(true);
+#endif
 #ifdef HAVE_GDIMAGEPNGEX
 EXTERN int   compression INIT(1);
 #endif
