@@ -1,21 +1,5 @@
 #include "dvipng.h"
 
-#if 0
-/*-->OpenFont*/
-/**********************************************************************/
-/**************************** OpenFont  *******************************/
-/**********************************************************************/
-void OpenFont(struct font_entry* tfontp)
-{
-  if (tfontp->filep != NULL)
-    return;         /* we need not have been called */
-  DEBUG_PRINT((DEBUG_DVI,"(OPEN %s) ", tfontp->name));
-  if ((tfontp->filep = fopen(tfontp->name,"rb")) == NULL) {
-    Warning("font file %s could not be opened", tfontp->name);
-  } 
-}
-#endif
-
 
 void CheckChecksum(uint32_t c1, uint32_t c2, const char* name)
 {
@@ -88,7 +72,8 @@ void FontDef(unsigned char* command, void* parent)
     DEBUG_PRINT((DEBUG_VF," %d %d '%.*s'",a,l,a+l,current+14));
 #ifdef DEBUG
   } else {
-    DEBUG_PRINT((DEBUG_DVI," %d %d %d %d %d %d '%.*s'",k,c,s,d,a,l,a+l,current+14));
+    DEBUG_PRINT((DEBUG_DVI," %d %d %d %d %d %d '%.*s'",k,c,s,d,a,l,
+		 a+l,current+14));
 #endif
   }
   if (a+l > STRSIZE-1)
@@ -171,12 +156,26 @@ void FontDef(unsigned char* command, void* parent)
 	     * ActualFactor(dvi->mag) * resolution * 5.0) + 0.5);
 }
 
+#ifdef HAVE_FT2
+inline char* kpse_find_ft(char* filename) 
+{
+    char* filepath = kpse_find_file(filename, kpse_type1_format, false);
+    if (filepath==NULL) 
+      filepath = kpse_find_file(filename, kpse_truetype_format, false);
+    return(filepath);
+}
+#endif
+
 void FontFind(struct font_entry * tfontptr)
 {
 #ifdef HAVE_LIBKPATHSEA
   kpse_glyph_file_type font_ret;
   char *name;
   unsigned dpi;  
+#ifdef HAVE_FT2
+  FT_Matrix *transform=NULL;
+  char *encoding=NULL,*psfile;
+#endif
 
   dpi = kpse_magstep_fix ((unsigned) (tfontptr->font_mag / 5.0 + .5),
 			resolution, NULL);
@@ -184,13 +183,36 @@ void FontFind(struct font_entry * tfontptr)
   DEBUG_PRINT((DEBUG_DVI,"\n  FIND FONT:\t%s %d",tfontptr->n,dpi));
 
   name = kpse_find_vf (tfontptr->n);
-  if (name) {
+  if (name!=NULL) {
     strcpy (tfontptr->name, name);
     free (name);
     InitVF(tfontptr);
-  } else {
+  }
+#ifdef HAVE_FT2
+  if (freetype && name==NULL) {
+    psfile = FindPSFontMap(tfontptr->n, &encoding, &transform);
+    if (psfile!=NULL) {
+      name = kpse_find_ft(psfile);
+      free(psfile);
+    } else
+      name = kpse_find_ft(tfontptr->n);
+    if (name!=NULL) {
+      strcpy (tfontptr->name, name);
+      free (name);
+      name = kpse_find_file(tfontptr->n, kpse_tfm_format, false);
+      if (name!=NULL) {
+	InitFT(tfontptr,dpi,encoding,transform);
+	if (transform!=NULL)
+	  free(transform);
+	ReadTFM(tfontptr,name);
+	free(name);
+      }
+    }
+  }
+#endif
+  if (name==NULL) {
     name = kpse_find_pk (tfontptr->n, dpi, &font_ret);
-    if (name) {
+    if (name!=NULL) {
       strcpy (tfontptr->name, name);
       free (name);
       
