@@ -1,4 +1,5 @@
 #include "dvipng.h"
+#include <libgen.h>
 
 static char *programname;
 static int   flags=BE_NONQUIET;
@@ -12,47 +13,40 @@ bool DecodeArgs(int argc, char ** argv)
   int     i;                 /* argument index for flags       */
   bool ppused=_FALSE;        /* Flag when -pp is used          */
   char *dviname=NULL;        /* Name of dvi file               */
-  char *outname=NULL;        /* Name of output file (concatenated
-				with the page number and .png) */
+  char *outname=NULL;        /* Name of output file            */
 
   programname=argv[0];
 
-  if (argc == 2 && (strcmp (argv[1], "--version") == 0)) {
-    puts (PACKAGE_STRING);
-#ifdef HAVE_LIBKPATHSEA
-    puts (KPSEVERSION);
-#endif
-    puts ("Copyright (C) 2002-2003 Jan-ke Larsson.\n\
-There is NO warranty.  You may redistribute this software\n\
-under the terms of the GNU General Public License.\n\
-For more information about these matters, see the files\n\
-named COPYING and dvipng.c.");
-    exit (EXIT_SUCCESS); 
-  }
-
   for (i=1; i<argc; i++) {
     if (*argv[i]=='-') {
       char *p=argv[i]+2 ;
       char c=argv[i][1] ;
+      if (c=='-') {
+	p++;
+	c=argv[i][2];
+      }
       switch (c) {
-#ifdef DEBUG
       case 'd':       /* selects Debug output */
+#ifdef DEBUG
 	{ 
 	  int debug = 0;
 	  
 	  if (*p == 0 && argv[i+1])
 	    p = argv[i+1];
 	  debug = atoi(p);
-	  flags |= (debug>0) ? debug * LASTFLAG * 2 : DEBUG_DVI;
+	  flags |= (debug>0) ? debug * DEBUG_DEFAULT : DEBUG_DVI;
 	  if (debug > 0 && p == argv[i+1])
 	    i++;
 #ifdef HAVE_LIBKPATHSEA
-	  kpathsea_debug = ( debug * LASTFLAG * 2) / LASTDEBUG;
+	  kpathsea_debug = ( debug * DEBUG_DEFAULT) / LASTDEBUG;
 #endif
 	  Message(PARSE_STDIN,"Debug output enabled\n");
 	}
-        break;
+#else
+	Warning("This instance of %s was compiled without the debug (-d) option",
+		programname);
 #endif
+        break;
       case 'o':       /* Output file is specified */
 	if (*p == 0 && argv[i+1])
 	  p = argv[++i] ;
@@ -78,8 +72,14 @@ named COPYING and dvipng.c.");
 	  if (argv[i+1])
 	    MFMODE = argv[++i] ;
 	  Message(PARSE_STDIN,"MetaFont mode: %s\n",MFMODE);
+	  break;
 	}
-	break;
+	goto DEFAULT;
+      case 'h':
+	if (strcmp(p,"elp") == 0 ) {
+	  break;
+	}
+	goto DEFAULT;
 #endif /* MAKETEXPK */
       case 'O' : /* Offset */
 	if (*p == 0 && argv[i+1])
@@ -143,6 +143,7 @@ named COPYING and dvipng.c.");
 	    Message(PARSE_STDIN,"Not caching images\n");
 	  break;
 	}
+	goto DEFAULT;
       case 'b':
 	if ( *p == 'g' ) { /* -bg background color */
 	  p++;
@@ -158,14 +159,27 @@ named COPYING and dvipng.c.");
 	  else 
 	    Message(PARSE_STDIN,"Transp. background (fallback rgb %d,%d,%d)\n",
 		    bRed,bGreen,bBlue);
+	  break;
 	} else if ( *p == 'd' ) { /* -bd border width */
 	  p++;
 	  if (*p == 0 && argv[i+1])
 	    p = argv[++i] ;
 	  borderwidth = atoi(p);
 	  Message(PARSE_STDIN,"Transp. border: %d dots\n",borderwidth);
+	  break;
+	} else if (strncmp(p,"aseline",7)==0) { /* Baseline reporting */ 
+	  if (p[8] != '0')
+	    flags |= REPORT_BASELINE;
+	  else
+	    flags &= !REPORT_BASELINE;
+	  break;
+	  if (flags & REPORT_BASELINE )
+	    Message(PARSE_STDIN,"Baseline reporting on\n",p);
+	  else 
+	    Message(PARSE_STDIN,"Baseline reporting off\n");
+	  break;
 	}
-	break;
+	goto DEFAULT;
       case 'f':
 	if ( *p == 'g' ) { /* -fg foreground color */
 	  p++;
@@ -173,13 +187,24 @@ named COPYING and dvipng.c.");
 	    p = argv[++i] ;
 	  resetcolorstack(p);
 	  Message(PARSE_STDIN,"Foreground: rgb %d,%d,%d\n",Red,Green,Blue);
+	  break;
+#ifdef HAVE_FT2
+	} else if ( strncmp(p,"reetype",7) == 0 ) { /* -freetype activation */
+	  freetype = (p[7] != '0');
+	  if (freetype)
+	    Message(PARSE_STDIN,"FreeType rendering on\n",p);
+	  else 
+	    Message(PARSE_STDIN,"FreeType rendering off\n");
+	  break;
+#endif
 	} else if (strcmp(p,"ollow") == 0 ) {
 	  if (DVIFollowToggle())
 	    Message(PARSE_STDIN,"Follow mode on\n");
 	  else
 	    Message(PARSE_STDIN,"Follow mode off\n");
+	  break;
 	}
-	break;
+	goto DEFAULT;
       case 'x' : case 'y' :
 	if (*p == 0 && argv[i+1])
 	  p = argv[++i] ;
@@ -198,7 +223,6 @@ named COPYING and dvipng.c.");
 	  Message(PARSE_STDIN,"Page list: %s\n",p);
 	  if (ParsePages(p))
 	    Fatal("bad page list specifier (-pp).");
-	  break ;
 	} else {   /* a -p specifier for first page */
 	  int32_t firstpage;
 	  bool abspage=_FALSE;
@@ -243,6 +267,22 @@ named COPYING and dvipng.c.");
 	  Message(PARSE_STDIN,"Normal order\n");
         break;
       case 'v':    /* verbatim mode */
+	if (strcmp(p, "ersion")==0) {
+	  if (strcmp(basename(programname),PACKAGE_NAME)!=0)
+	    printf("%s (%s) %s\n",basename(programname),
+		   PACKAGE_NAME,PACKAGE_VERSION);
+	  else
+	    puts(PACKAGE_STRING);
+#ifdef HAVE_LIBKPATHSEA
+	  puts (KPSEVERSION);
+#endif
+	  puts ("Copyright (C) 2002-2003 Jan-ke Larsson.\n\
+There is NO warranty.  You may redistribute this software\n\
+under the terms of the GNU General Public License.\n\
+For more information about these matters, see the files\n\
+named COPYING and dvipng.c.");
+	  exit (EXIT_SUCCESS); 
+	}
 	if (*p != '0')
 	  flags |= BE_NONQUIET | BE_VERBOSE;
 	else
@@ -272,7 +312,7 @@ named COPYING and dvipng.c.");
       case '\0':
 	flags |= PARSE_STDIN;
 	break;
-      default:
+      DEFAULT: default:
 	Warning("%s is not a valid option", argv[i]);
       }
     } else {
@@ -290,37 +330,40 @@ named COPYING and dvipng.c.");
   }
 
   if (dvi==NULL) {
-    fprintf(stdout,"\nUsage: dvipng [OPTION]... FILENAME[.dvi]\n");
+    fprintf(stdout,"\nUsage: %s [OPTION]... FILENAME[.dvi]\n", programname);
     fprintf(stdout,"Options are chosen to be similar to dvips' options where possible:\n");
 #ifdef DEBUG
-    fprintf(stdout,"  -d #       Debug (# is the debug bitmap, 1 if not given)\n");
+    fprintf(stdout,"  -d #         Debug (# is the debug bitmap, 1 if not given)\n");
 #endif
-    fprintf(stdout,"  -D #       Resolution\n");
-    fprintf(stdout,"  -M*        Don't make fonts\n");
-    fprintf(stdout,"  -l #       Last page to be output\n");
-    fprintf(stdout,"  -mode s    MetaFont mode (default cx)\n");
-    fprintf(stdout,"  -o f       Output file, '%%d' is pagenumber\n");
-    fprintf(stdout,"  -O c       Image offset\n");
-    fprintf(stdout,"  -p #       First page to be output\n");
-    fprintf(stdout,"  -pp #,#..  Page list to be output\n");
-    fprintf(stdout,"  -q*        Quiet operation\n");
-    fprintf(stdout,"  -r*        Reverse order of pages\n");
-    fprintf(stdout,"  -t c       Paper format (also accepts e.g., '-t a4')\n");
-    fprintf(stdout,"  -T c       Image size (also accepts '-T bbox' and '-T tight')\n");
-    fprintf(stdout,"  -v*        Verbose operation\n");
-    fprintf(stdout,"  -x #       Override dvi magnification\n");
-    fprintf(stdout,"  -          Interactive query of options\n");
+    fprintf(stdout,"  -D #         Resolution\n");
+    fprintf(stdout,"  -M*          Don't make fonts\n");
+    fprintf(stdout,"  -l #         Last page to be output\n");
+    fprintf(stdout,"  -mode s      MetaFont mode (default cx)\n");
+    fprintf(stdout,"  -o f         Output file, '%%d' is pagenumber\n");
+    fprintf(stdout,"  -O c         Image offset\n");
+    fprintf(stdout,"  -p #         First page to be output\n");
+    fprintf(stdout,"  -pp #,#..    Page list to be output\n");
+    fprintf(stdout,"  -q*          Quiet operation\n");
+    fprintf(stdout,"  -r*          Reverse order of pages\n");
+    fprintf(stdout,"  -t c         Paper format (also accepts e.g., '-t a4')\n");
+    fprintf(stdout,"  -T c         Image size (also accepts '-T bbox' and '-T tight')\n");
+    fprintf(stdout,"  -v*          Verbose operation\n");
+    fprintf(stdout,"  -x #         Override dvi magnification\n");
+    fprintf(stdout,"  -            Interactive query of options\n");
     fprintf(stdout,"\nThese do not correspond to dvips options:\n");
-    fprintf(stdout,"  -bd #      Transparent border width in dots\n");
-    fprintf(stdout,"  -bg s      Background color (TeX-style color)\n");
-    fprintf(stdout,"  -fg s      Foreground color (TeX-style color)\n");
-    fprintf(stdout,"  -follow    Follow mode\n");
-#ifdef HAVE_GDIMAGECREATETRUECOLOR
-    fprintf(stdout,"  -truecolor Truecolor output\n");
+    fprintf(stdout,"  -bd #        Transparent border width in dots\n");
+    fprintf(stdout,"  -bg s        Background color (TeX-style color)\n");
+    fprintf(stdout,"  -fg s        Foreground color (TeX-style color)\n");
+    fprintf(stdout,"  --follow*    Follow mode\n");
+#ifdef HAVE_FT2
+    fprintf(stdout,"  --freetype*  FreeType font rendering (default on)\n");
 #endif
-    fprintf(stdout,"  -Q #       Quality (~xdvi's shrinkfactor)\n");
+#ifdef HAVE_GDIMAGECREATETRUECOLOR
+    fprintf(stdout,"  --truecolor* Truecolor output\n");
+#endif
+    fprintf(stdout,"  -Q #         Quality (~xdvi's shrinkfactor)\n");
 #ifdef HAVE_GDIMAGEPNGEX
-    fprintf(stdout,"  -z #       PNG compression level\n");
+    fprintf(stdout,"  -z #         PNG compression level\n");
 #endif
     
     fprintf(stdout,"\n   # = number   f = file   s = string  * = suffix, '0' to turn off\n");
@@ -408,11 +451,12 @@ void Fatal (char *fmt, ...)
   va_list args;
 
   va_start(args, fmt);
-  fprintf(ERR_STREAM, "\n");
-  fprintf(ERR_STREAM, "%s: Fatal error, ", programname);
-  vfprintf(ERR_STREAM, fmt, args);
+  fflush(stdout);
+  fprintf(stderr, "\n");
+  fprintf(stderr, "%s: Fatal error, ", programname);
+  vfprintf(stderr, fmt, args);
 
-  fprintf(ERR_STREAM, "\n\n");
+  fprintf(stderr, "\n\n");
   va_end(args);
   exit(EXIT_FAILURE);
 }
@@ -430,9 +474,10 @@ void Warning(char *fmt, ...)
   va_start(args, fmt);
 
   if ( flags & BE_NONQUIET ) {
-    fprintf(ERR_STREAM, "%s warning: ", programname);
-    vfprintf(ERR_STREAM, fmt, args);
-    fprintf(ERR_STREAM, "\n");
+    fflush(stdout);
+    fprintf(stderr, "%s warning: ", programname);
+    vfprintf(stderr, fmt, args);
+    fprintf(stderr, "\n");
     va_end(args);
   }
 }
@@ -449,7 +494,8 @@ void Message(int activeflags, char *fmt, ...)
   if ( flags & activeflags ) {
     vfprintf(stdout, fmt, args);
 #ifdef DEBUG
-    fflush(stdout);
+    if (flags & (LASTDEBUG-LASTFLAG)*2)
+      fflush(stdout);
 #endif
   }
   va_end(args);
